// this module checks errors for two patterns: Counter and Running1s
// The check is done comparing 2 following data, so the first data
// will typically flagged as error.
module ErrorChecker  ( 
 input  TTC_CK2x, reset,
 input  [8:0]  SYNC1, SYNC2, SYNC3, SYNC4, 
               SYNC5, SYNC6, SYNC7, SYNC8,  
 output reg [8:1]  InstantCntErr, InstantR1sErr,
 output reg [15:0] AccCntErr, AccR1sErr         );

 reg [8:0] SYNC1D, SYNC2D, SYNC3D, SYNC4D, SYNC5D, SYNC6D, SYNC7D, SYNC8D,
           SYNC1Expected, SYNC2Expected, SYNC3Expected, SYNC4Expected,
           SYNC5Expected, SYNC6Expected, SYNC7Expected, SYNC8Expected;
 
 always @ (posedge TTC_CK2x) begin 
    // calculate expected value (there is also a 1-tick delay) 
  SYNC1Expected <= SYNC1 + 9'b1;  SYNC2Expected <= SYNC2 + 9'b1;  
  SYNC3Expected <= SYNC3 + 9'b1;  SYNC4Expected <= SYNC4 + 9'b1;  
  SYNC5Expected <= SYNC5 + 9'b1;  SYNC6Expected <= SYNC6 + 9'b1;  
  SYNC7Expected <= SYNC7 + 9'b1;  SYNC8Expected <= SYNC8 + 9'b1;  
    // verify 
  if (SYNC1Expected == SYNC1) InstantCntErr[1] <= 0; else InstantCntErr[1] <= 1; 
  if (SYNC2Expected == SYNC2) InstantCntErr[2] <= 0; else InstantCntErr[2] <= 1; 
  if (SYNC3Expected == SYNC3) InstantCntErr[3] <= 0; else InstantCntErr[3] <= 1; 
  if (SYNC4Expected == SYNC4) InstantCntErr[4] <= 0; else InstantCntErr[4] <= 1; 
  if (SYNC5Expected == SYNC5) InstantCntErr[5] <= 0; else InstantCntErr[5] <= 1; 
  if (SYNC6Expected == SYNC6) InstantCntErr[6] <= 0; else InstantCntErr[6] <= 1; 
  if (SYNC7Expected == SYNC7) InstantCntErr[7] <= 0; else InstantCntErr[7] <= 1; 
  if (SYNC8Expected == SYNC8) InstantCntErr[8] <= 0; else InstantCntErr[8] <= 1; 

  // register to compare subsequent samples 
  SYNC1D <= SYNC1;  SYNC2D <= SYNC2;  SYNC3D <= SYNC3;  SYNC4D <= SYNC4;  
  SYNC5D <= SYNC5;  SYNC6D <= SYNC6;  SYNC7D <= SYNC7;  SYNC8D <= SYNC8;  
  if ({SYNC1D[7:0],SYNC1D[8]}== SYNC1) InstantR1sErr[1] <= 0; else InstantR1sErr[1] <= 1; 
  if ({SYNC2D[7:0],SYNC2D[8]}== SYNC2) InstantR1sErr[2] <= 0; else InstantR1sErr[2] <= 1; 
  if ({SYNC3D[7:0],SYNC3D[8]}== SYNC3) InstantR1sErr[3] <= 0; else InstantR1sErr[3] <= 1; 
  if ({SYNC4D[7:0],SYNC4D[8]}== SYNC4) InstantR1sErr[4] <= 0; else InstantR1sErr[4] <= 1; 
  if ({SYNC5D[7:0],SYNC5D[8]}== SYNC5) InstantR1sErr[5] <= 0; else InstantR1sErr[5] <= 1; 
  if ({SYNC6D[7:0],SYNC6D[8]}== SYNC6) InstantR1sErr[6] <= 0; else InstantR1sErr[6] <= 1; 
  if ({SYNC7D[7:0],SYNC7D[8]}== SYNC7) InstantR1sErr[7] <= 0; else InstantR1sErr[7] <= 1; 
  if ({SYNC8D[7:0],SYNC8D[8]}== SYNC8) InstantR1sErr[8] <= 0; else InstantR1sErr[8] <= 1; 
 end   //  always
 
 always @ (posedge TTC_CK2x) begin 
  if (reset) begin  // reset  
        AccCntErr <= 0;   AccR1sErr <= 0; 
  end   // reset
  else begin
   // accumulate the two types of errors
  if ((|InstantCntErr) & (AccCntErr != 16'hFFFF)) AccCntErr <= AccCntErr + 1'b1;
  if ((|InstantR1sErr) & (AccR1sErr != 16'hFFFF)) AccR1sErr <= AccR1sErr + 1'b1;
  end 
 end   //  always
endmodule