/*
30-June-2005  Drew Baden
1. Start with s2 version of the sandwich self-triggering board....
2. Modify so that each Xilinx in the HTR sends, at 80MHz
    o 2 bits per channel (24 channels, 48 bits)
	 o 16 bits for sumET
	 o 1 bit for "frame"
	 o 1 bit for L1A
  Total: 66 bits @ 80MHz
3. We need some output pins to debug, so we'll use the "Wesley" connectors:
  J4 and J5 are the 50pin connectors that are available and look like this
  on the board:
  
     J4          J5         J7
  -50- -25-   -50- -25-    |||||
  -49- -24-   -49- -24-    |||||
  
  -27- -02-   -27- -02-    |||||
  -26- -01-   -26- -01-    |||||
                            J6
on J4 and J5:  pin 1 is the bottom right pin in the 25row by 2 columns

On J4, there are 23 free pins
On J5, there are 23 free pins

Pins 49 and 50 on J4 are REFCLK+ and REFCLK- driven from the FPGA
on B12.  The firmware below drives this pin (B12) from the DCM through
a TTL->LVPECL converter (U2A).  The origin of this clock is the 40MHz
clock from the TTCrx

We need an 80MHz clock to latch things.  The DCM produces a 80MHz clock
  which is 2x the TTC clock.  Drive this onto J5/pin1 from U3 (CK80)

Data from each HTR Xilinx:

*/  

`include "define.v"
module SandwichTop 
 ( 
 input  CLKp, CLKn, // 40MHz differential RX_CLK
 input  BCOp, BCOn, // RX_BC0 common to all SLB boards
 input  SLB_ID,
 input  TTC_CK40Des1, TTC_BRCST_STRB1, //TTC_BCR,
 input  SCLK,       // 80 MHz Clock from onboard crystal
 input  LOC_nCS,  LocWRITE_b, 
 input  [7:2] TTC_BRCST,
 input [23:0] TOP_D, BOT_D,
 input [11:0] TOPbus, BOTbus,      
 input  [`LB_Addr_Size-1:0]  LocAdd,  // The Address bus.
 inout  [`REG_WIDTH-1:0]     LocBus,  // The bidir Data bus 
 output SLB_CFGDONE,
 output [22:0] SPY4,		// ouput onto J4  see http://www.physics.umd.edu/hep/HTR/luminosity/uwspy.html
 output [22:0] SPY5,		// ouput onto J5                            ditto

 output LVDS1n, LVDS1p, LVDS2n, LVDS2p,// to MDR14 J10
 output REFCLK, TRSTN,  LED_CONTROL          );

 wire  VME_Hard_rst, VME_Soft_rst, VME_Start, VME_Stop, VME_LVDSout,
       run_status, BunchEr, GateTTCcommands, Hard_rst, Soft_rst;
 wire [15:0] TTC_test0, TTC_test1, TTC_test2, TTC_test3,
             TTC_test4, TTC_test5, TTC_test6, TTC_test7,
             TTC_test8, TTC_test9, TTC_testA, TTC_testB,
             TTC_testC, TTC_testD, TTC_testE, TTC_testF;

 wire [8:1]  TPSpyFifoEmpty, TPSpyFifoFull, TPSpyFifoRd,
             InstantCntErr,  InstantR1sErr;
 wire [15:0] AccCntErr, AccR1sErr;  
 wire [35:0]  TPSpyFIFO1_out, TPSpyFIFO2_out;
 reg VME_LVDSout_D, LVDSout1, LVDSout2;
 wire [7:0] LastTTCcommand;
 reg [23:0] rTOP_D, rBOT_D;
 reg [11:0] rTOPbus, rBOTbus;
 reg [15:0] RxClkCnt, RxBC0Cnt, TClkCnt;


//  ********************************************************************
//      ********************  CLOCK SECTION    *****************
//            *********************************************

 wire   RxClk_Buf, SCLK_buf;
 wire   Tclk0, TTC_CK, Tclk2x_dll, TTC_CK2x, TTC_DLL_lock;
 reg    [2:0] UnlockCnt; // counter for proper reset of DLL 

 // Instantiate the Differential Global Input Buffer 
  IBUFGDS_LVPECL_25 difclkbuf (.I(CLKp), .IB(CLKn), .O(RxClk_Buf));
  IBUFG sclkbuf ( .I(SCLK), .O(SCLK_buf) );
                  
// Generate a DLL reset (3-ticks) after FPGA is configured, see:
// www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
  FDS flop1 (.D(1'b0), .C(SCLK_buf), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(SCLK_buf), .Q(out2)); 
  FD  flop3 (.D(out2), .C(SCLK_buf), .Q(out3)); 
  FD  flop4 (.D(out3), .C(SCLK_buf), .Q(out4)); 
  assign ConfigRst = (out2 | out3 | out4); 

  DCM TTC_DCM
   (.CLKIN(TTC_CK40Des1), .CLKFB(TTC_CK),  .RST(ConfigRst),  
    .CLK0(Tclk0),   .CLK90(),  .CLK180(),  .CLK270(),	
    .CLK2X(),  .CLK2X180(Tclk2x_dll),  // here u can use the inverted clock.
    .CLKDV(),   .CLKFX(),   .CLKFX180(),
    .LOCKED(TTC_DLL_lock),  .STATUS(),
    .DSSEN(1'b0),
    .PSCLK(1'b0),  .PSEN(1'b0),  .PSINCDEC(1'b0), .PSDONE() );

  BUFG TCK40BUF (.I(Tclk0), .O(TTC_CK));  
  BUFG TCK2XBUF (.I(Tclk2x_dll), .O(TTC_CK2x)); 


//       *************  END OF CLOCK SECTION  *****************
//  *****************************************************************
// **** Register, Spy and Check the HTR data ***

//
//register data since it is arriving at 80MHz
//
 always @ (posedge TTC_CK2x) begin 
  rTOP_D <= TOP_D;
  rBOT_D <= BOT_D;
  rTOPbus <= TOPbus;
  rBOTbus <= BOTbus;
  end
reg[26:0] clk_scaled;
//
// spy output onto wisconsin receiver
//	see http://www.physics.umd.edu/hep/HTR/luminosity/uwspy.html
// there are 23 pins available per wisconsin connectors J4 and J5
//                2      1       1          1                6        12
assign SPY4 = {2'b01, TTC_CK, TTC_CK2x, TTC_BRCST_STRB1, TTC_BRCST, TOPbus};
//assign SPY4 = {clk_scaled[22:0]};
assign SPY5 = {2'b10, TTC_CK, TTC_CK2x, TTC_BRCST_STRB1, TTC_BRCST, rTOPbus};
//assign SPY5 = {clk_scaled[22:0]};

always @ (posedge TTC_CK2x) clk_scaled <= #1 clk_scaled + 1;
assign LED_CONTROL = clk_scaled[25];

//LED_Sine breathing(.clock(SCLK_buf), .nReset(1'b1), .led(LED_CONTROL));
 assign REFCLK = TTC_CK;
 assign SLB_CFGDONE = SLB_ID; // in case ECAL TCC uses this board
 assign TRSTN = 1'b1; // Wisc. Rec_Mezz JTAG reset.
 IBUFDS_LVPECL_25 RxBC0buf (.I(BCOp), .IB(BCOn), .O(RxBC0));
 OBUFDS_LVDS_25 pair1 (.I(LVDSout1), .O(LVDS1p), .OB(LVDS1n));
 OBUFDS_LVDS_25 pair2 (.I(LVDSout2), .O(LVDS2p), .OB(LVDS2n));
//
// decode the TTC to keep track (just for checking)
//
decodeTTC TTCdecode (
 .TTC_CK40Des1(TTC_CK),
 .reset(VME_Hard_rst), 
 .TTC_Brcst(TTC_BRCST),
 .TTC_BrcstStr(TTC_BRCST_STRB1),   
 .TTC_test0(TTC_test0), .TTC_test1(TTC_test1),
 .TTC_test2(TTC_test2), .TTC_test3(TTC_test3),
 .TTC_test4(TTC_test4), .TTC_test5(TTC_test5),
 .TTC_test6(TTC_test6), .TTC_test7(TTC_test7),
 .TTC_test8(TTC_test8), .TTC_test9(TTC_test9),
 .TTC_testA(TTC_testA), .TTC_testB(TTC_testB),
 .TTC_testC(TTC_testC), .TTC_testD(TTC_testD),
 .TTC_testE(TTC_testE), .TTC_testF(TTC_testF),
 .LastTTCcommand(LastTTCcommand)
 );

always @ (posedge TTC_CK) begin
  LVDSout1 <= VME_LVDSout;
  LVDSout2 <= VME_LVDSout;
  end

fifo_ic_v2  TPSpyFIFO1 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({rTOPbus, rTOP_D}), 
   .write_clock_in(TTC_CK2x),              .write_enable_in(run_status),
   .read_clock_in(TTC_CK),               .read_enable_in(TPSpyFifoRd[1]),
   .read_data_out({TPSpyFIFO1_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[1]),            .empty_out(TPSpyFifoEmpty[1])  );

fifo_ic_v2  TPSpyFIFO2 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({rBOTbus, rBOT_D}), 
   .write_clock_in(TTC_CK2x),              .write_enable_in(run_status),
   .read_clock_in(TTC_CK),               .read_enable_in(TPSpyFifoRd[2]),
   .read_data_out({TPSpyFIFO2_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[2]),            .empty_out(TPSpyFifoEmpty[2])  );


////////////////////////////////////////////////////////


LocBusSlave LocBusSlave (
    .clk(TTC_CK),      .rst(1'b0), 
    .LocAddr(LocAdd),    .LocData(LocBus), 
    .LocCS_b(LOC_nCS),   .LocWRITE_b(LocWRITE_b), 
    .LB_STATE(),   .Din(), .Dout(),      .Tri(), 
    .RxDLL_lock(1'b0), .TTC_DLL_lock(TTC_DLL_lock), .UnlockCnt(UnlockCnt),
    .DLL_status(16'b0),
    .LastTTCcommand(LastTTCcommand), .GateTTCcommands(GateTTCcommands), 
    .VME_Hard_rst(VME_Hard_rst),     .VME_Soft_rst(VME_Soft_rst), 
    .VME_Start(VME_Start),           .VME_Stop(VME_Stop), 
    .VME_LVDSout(VME_LVDSout), 
    .run_status(run_status),         
//    .InstantCntErr(InstantCntErr),   .InstantR1sErr(InstantR1sErr), 
//    .AccCntErr(AccCntErr),           .AccR1sErr(AccR1sErr),
    .RxClkCnt(RxClkCnt), .RxBC0Cnt(RxBC0Cnt),
    .TClkCnt(TClkCnt), .TxBC0Cnt(16'b0),
    .TPSpyFifoRd(TPSpyFifoRd),
    .TPSpyFifoEmpty(TPSpyFifoEmpty),   .TPSpyFifoFull(TPSpyFifoFull), 
    .TPSpyFIFO1_out(TPSpyFIFO1_out),   .TPSpyFIFO2_out(TPSpyFIFO2_out),
//    .TPSpyFIFO3_out(TPSpyFIFO3_out),   .TPSpyFIFO4_out(TPSpyFIFO4_out),
//    .TPSpyFIFO5_out(TPSpyFIFO5_out),   .TPSpyFIFO6_out(TPSpyFIFO6_out),
//    .TPSpyFIFO7_out(TPSpyFIFO7_out),   .TPSpyFIFO8_out(TPSpyFIFO8_out),
    .TTC_test0(TTC_test0),       .TTC_test1(TTC_test1), 
    .TTC_test2(TTC_test2),       .TTC_test3(TTC_test3), 
    .TTC_test4(TTC_test4),       .TTC_test5(TTC_test5), 
    .TTC_test6(TTC_test6),       .TTC_test7(TTC_test7), 
    .TTC_test8(TTC_test8),       .TTC_test9(TTC_test9), 
    .TTC_testA(TTC_testA),       .TTC_testB(TTC_testB), 
    .TTC_testC(TTC_testC),       .TTC_testD(TTC_testD), 
    .TTC_testE(TTC_testE),       .TTC_testF(TTC_testF), 
    .BunchEr(BunchEr)
        );

 commander commander (  
  .clk(SCLK_buf),    .GateTTCcommands(GateTTCcommands),
  .TTC_Hard_rst(1'b0), .TTC_Soft_rst(1'b0),
  .TTC_Start(1'b0), .TTC_Stop(1'b0), 
  .VME_Hard_rst(VME_Hard_rst), .VME_Soft_rst(VME_Soft_rst), 
  .VME_Start(VME_Start), .VME_Stop(VME_Stop), 
  .Hard_rst(Hard_rst),  .Soft_rst(Soft_rst), 
  .run_status(run_status)       );

 always @ (posedge RxClk_Buf) begin 
   if (Soft_rst) begin
      RxClkCnt <= 0;  RxBC0Cnt <= 0;
   end
   else if (run_status) begin
      RxClkCnt <= RxClkCnt + 1;
      if (RxBC0) RxBC0Cnt <= RxBC0Cnt + 1;
   end
 end // always

 always @ (posedge TTC_CK) begin 
   if (Soft_rst)         TClkCnt <= 0; 
   else if (run_status)  TClkCnt <= TClkCnt + 1;
 end // always

endmodule